local silv_configfilename = "client/silverconfig.dat"

local paramnames = {
"silv_auracolor",
"silv_invertflight",
"silv_throwall",
"silv_norollangle",
"silv_memevoices"
}

local loccvars = {}	-- local cvars.

local silverparams = {
	["silv_auracolor"] = "Aqua",
	["silv_invertflight"] = "Off",
	["silv_throwall"] = "Off",
	["silv_norollangle"] = "Off",
	["silv_memevoices"] = "Off",
}

local loaded
-- don't write params during the initial command loading.

-- writes all of this into its own file
local function writeSilverParams()
	if not loaded return end
	--CONS_Printf(consoleplayer, "Writing Silver client commands into \'luafiles/"..silv_configfilename.."\'")
	-- fetch cvar values:
	for i = 1, #loccvars
		local cv = loccvars[i]
		silverparams[cv.name] = cv.string or tostring(cv.value)	-- blergh?
	end

	local f = io.openlocal(silv_configfilename, "w")
	if not f
		CONS_Printf(consoleplayer, "\x82WARNING:\x80: Could not create \'silverconfig.dat\'")
		-- SRB2 probably already has error handling for this, just in case.
		return
	end	-- !?

	for i = 1, #paramnames
		f:write(paramnames[i].." "..(silverparams[paramnames[i]] or "0").."\n")
	end
	--CONS_Printf(consoleplayer, "(Write succesful)")
	f:close()
end

local function updateSilverParam(param, value)
	silverparams[param] = value
end

-- silver aura color:
rawset(_G, "silv_auracolor", function(p)

	if not (netgame or splitscreen)
		return SKINCOLOR_AQUA
	end

	if G_GametypeHasTeams()
		return p.skincolor
	end

	if p.silv_auracolor == -1
		return ColorOpposite(p.skincolor)
	elseif p.silv_auracolor == nil or p.silv_auracolor <= 0 or p.silv_auracolor >= #skincolors
		return SKINCOLOR_AQUA
	else
		return p.silv_auracolor
	end
end)

COM_AddCommand("silv_auracolor", function(p, arg)
	if arg == nil
		CONS_Printf(p, "\x82".."silv_auracolor <number / Colorname>".."\x80"..": Change your aura color when playing as Silver. Set to \'-1\' or \'Opposite\' for opposite color.")
		return
	end

	local args = arg:lower()
	local argn = tonumber(arg)

	if args == "opposite"
		p.silv_auracolor = -1	-- easier for me
		if p == consoleplayer
			updateSilverParam("silv_auracolor", p.silv_auracolor)
			writeSilverParams()	-- bit of a hack here
		end
		return
	end

	if argn
		if (argn > 0 and argn < #skincolors) or argn == -1
			p.silv_auracolor = argn
		else
			CONS_Printf(p, "Color "..tostring(argn).." not in range")
		end
		if p == consoleplayer
			updateSilverParam("silv_auracolor", p.silv_auracolor)
			writeSilverParams()
		end
		return
	end

	if R_GetColorByName(args)
		p.silv_auracolor = R_GetColorByName(args)
	end
	if p == consoleplayer
		updateSilverParam("silv_auracolor", p.silv_auracolor)
		writeSilverParams()
	end
end
)

COM_AddCommand("silv_invertflight", function(p, arg)
	if arg == nil
		CONS_Printf(p, "\x82".."silv_invertflight <On/Off>".."\x80"..": If turned On, inverts the Psycho Boost's vertical commands.")
		return
	end

	local args = arg:lower()
	local argn = tonumber(arg)

	if args == "on"
	or argn == 1
		p.silv_invertflight = true
	elseif args == "off"
	or argn == 0
		p.silv_invertflight = nil
	else
		CONS_Printf(p, "silv_invertflight: Invalid argument \'"..arg.."\'. Please use \'0\'/\'1\' or \'On\'/\'Off\'")
	end
	if p == consoleplayer
		updateSilverParam("silv_invertflight", p.silv_inverflight)
		writeSilverParams()
	end
end
)

COM_AddCommand("silv_throwall", function(p, arg)
	if arg == nil
		CONS_Printf(p, "\x82".."silv_throwall <On/Off>".."\x80"..": If turned On, you will default to throwing objects on all enemies at once.")
		return
	end

	local args = arg:lower()
	local argn = tonumber(arg)

	if args == "on"
	or argn == 1
		p.silv_throwall = true
	elseif args == "off"
	or argn == 0
		p.silv_throwall = nil
	else
		CONS_Printf(p, "silv_throwall: Invalid argument \'"..arg.."\'. Please use \'0\'/\'1\' or \'On\'/\'Off\'")
	end

	if p == consoleplayer
		updateSilverParam("silv_throwall", p.silv_throwall)
		writeSilverParams()
	end
end
)

rawset(_G, "silv_norollangle", CV_RegisterVar({
	name = "silv_norollangle",
	defaultvalue = 0,
	flags = CV_SAVE|CV_CALL,
	func = writeSilverParams,
	possiblevalue = CV_OnOff,
}))

local meme = CV_RegisterVar({
	name = "silv_memevoices",
	defaultvalue = 0,
	flags = CV_SAVE|CV_CALL,
	func = writeSilverParams,
	possiblevalue = CV_OnOff
})
rawset(_G, "silv_meme", function() return meme.value end)

loccvars = {silv_norollangle, meme}
-- DON'T FORGET TO ADD CVARS HERE FOR I/O.

local configplayer	-- used to know when the config should be reloaded
rawset(_G, "silv_config", function()
	if consoleplayer and consoleplayer.valid and (not configplayer or not configplayer.valid)
		configplayer = consoleplayer
		loaded = nil	-- don't write params here, that's needelessly using the disk.
		-- config must be reloaded
		-- read silverconfig.cfg from luafiles!
		local f = io.openlocal(silv_configfilename, "r")
		if not f
			loaded = true	-- consider that we succesfully loaded the file.
			return
		end	-- file doesn't exist yet. It will be created if you change parameters.
		CONS_Printf(consoleplayer, "Loading Silver client commands from \'luafiles/"..silv_configfilename.."\'")
		local line = f:read()
		while (line)
			COM_BufInsertText(consoleplayer, line)	-- execute whatever the fuck is written.
			line = f:read()	-- ready next line.
		end

		f:close()

		--COM_BufInsertText(consoleplayer, "exec silverconfig.cfg -noerror")	-- for some retarded ass reason, you can't use both -silent and -noerror
		loaded = true
	end
end)