-- Shield thinkers for Silver.

-- List of thinkers are:
/*
	-boostThink 	(mobj_t mo)		-> runs during boost
	-boostDamage 	(boolean)		-> if false, then the boost isn't cancelled on damage
	-boostBump		(mobj_t mo)		-> runs when a wall gets bumped
	-boostStart		(mobj_t mo)		-> runs when the boost has just begun
	-floorTouch		(mobj-t mo)		-> runs when you just hit the ground while boosting
	-boostSlideFire	(boolean)		-> if true, spawns a fire trail
	-boostWater		(boolean)		-> if true, immune to water slowdown
	-boostSneaker 	(boolean)		-> if true, the boost behaves as if you had super sneakers / super form
	-ico			(string lump)	-> icon

	- downwardsCap	(int)			-> max downwards momentum
	- upwardsCap	(int)			-> max upwards momentum
	- downwardsSpeed(int)			-> max downwards speed
	- upwardsSpeed  (int)			-> max upwards speed
*/

local whirltable = {
	upwardsCap = 24,
	boostStart =	function(mo)
						mo.silv_boostt = 0
						mo.silv_boostsoundhelper = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK)
						mo.silv_boostsoundhelper.state = S_INVISIBLE
						mo.silv_boostsoundhelper.fuse = 5
					end,
	boostThink = 	function(mo)

						if not mo.silv_boostsoundhelper or not mo.silv_boostsoundhelper.valid return end	-- !?

						mo.silv_boostsoundhelper.fuse = 5
						silv_tp(mo.silv_boostsoundhelper, mo.x, mo.y, mo.z)
						if mo.silv_boostt % 70 == 0
							S_StartSound(mo.silv_boostsoundhelper, sfx_s3kcel)
						end
						mo.silv_boostt = $+1
					end,
	ico = "ICO_WIND",
	icooffs = {-9, 0},
}
silv_addShieldStuff(SH_WHIRLWIND, whirltable)

local boomtable = {

	boostBump =			function(mo)
							P_BlackOw(mo.player)
							mo.player.powers[pw_shield] = 0
							local ang = mo.angle
							P_DoPlayerPain(mo.player)
							mo.angle = ang
							P_InstaThrust(mo, ang, -10*mo.scale)
							mo.player.drawangle = ang
						end,

	boostStart =		function(mo)

							-- spawn visual:
							local h_an = mo.angle + ANG1*90
							for i = 1, 16

								local v_an = 360/16 * ANG1* i

								-- honestly? I hate maths.
								local pos = FixedMul(mo.scale, sin(v_an)*32)
								local tx = mo.x + FixedMul(cos(h_an), pos)
								local ty = mo.y + FixedMul(sin(h_an), pos)
								local tz = mo.z + FixedMul(mo.scale, 32*cos(v_an))

								local momx = mo.x - tx
								local momy = mo.y - ty
								local momz = mo.z - tz

								local spark = P_SpawnMobj(mo.x, mo.y, mo.z, MT_SUPERSPARK)
								spark.momx = momx
								spark.momy = momy
								spark.momz = momz
								spark.scale = mo.scale
							end

							mo.silv_fspeed = 95*mo.scale	-- the blast makes you go at full throttle from the get go!
						end,

	ico = "ICO_NUKE",
	icooffs = {-13, -6}
}
silv_addShieldStuff(SH_ARMAGEDDON, boomtable)

local forcetable = {
	--boostDamage = false,

	boostStart = function(mo)
					mo.silv_useheld = 2
					mo.silv_momhalt = nil
				end,

	boostThink = 	function(mo)
						local p = mo.player

						if p.cmd.buttons & BT_USE
							mo.silv_useheld = $+1
						else
							mo.silv_useheld = 0
						end

						if mo.silv_useheld == 1
							mo.silv_momhalt = true
							S_StartSound(mo, sfx_ngskid)
							S_StartSound(mo, sfx_spin)
							local stop = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_GHOST)
							stop.scale = FixedMul(mo.scale, skins[mo.skin].shieldscale)	-- bruh!
							stop.state = S_FORC21
							stop.destscale = stop.scale << 1
							stop.scalespeed = FixedMul(stop.scalespeed, stop.scale)
							stop.height = 32*stop.scale
							stop.fuse = 10
							stop.flags = $ | MF_NOCLIPHEIGHT
							--stop.momz = mo.momz
						end

						if mo.silv_momhalt
							mo.momx = 0
							mo.momy = 0
							mo.momz = 0

							mo.state = S_PLAY_ROLL
							mo.frame = leveltime%6	-- for some reason it won't animate otherwise?
							mo.silv_energy = $+5 + (leveltime & 1 and 1 or 0)	-- you lose 6 normally, so it's like a normal float??

							-- show indicators:
							if leveltime%5 == 0
								local s = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_THOK)
								s.scale = $*3/2
								s.color = silv_auracolor(p)
								P_InstaThrust(s, mo.angle, mo.silv_fspeed)

								local invert = p.silv_invertflight and -1 or 1
								s.momz = mo.silv_fmomz*mo.scale * invert * P_MobjFlip(mo)
							end

							if not (p.cmd.buttons & BT_USE)
								mo.silv_momhalt = nil
								S_StartSound(mo, sfx_psybst)
							end	-- resume flight!
						end
					end,

	ico = "ICO_SHLD",
	icooffs = {-11, -9},
}
silv_addShieldStuff(SH_FORCE, forcetable)

local elementaltable = {
	boostWater = true,
	boostSlideFire = true,
	downwardsCap = -90,
	downwardsSpeed = 6,
	ico = "ICO_ELEM",

	boostThink = 	function(mo)

						local fire
						if mo.eflags & MFE_VERTICALFLIP
							if mo.momz > 10*mo.scale
								fire = true
							end
						else
							if mo.momz < -10*mo.scale
								fire = true
							end
						end

						mo.silv_elmfire = fire	-- buffer this.

						if fire
							if not (mo.player.pflags & PF_SHIELDABILITY)
								S_StartSound(mo, sfx_s3k43)
							end
							mo.player.pflags = $|PF_SHIELDABILITY
						else
							mo.player.pflags = $ & ~PF_SHIELDABILITY
						end
					end,

	floorTouch =	function(mo)

						if mo.silv_elmfire

							mo.momz = 0
							P_InstaThrust(mo, mo.angle, mo.silv_fspeed or mo.player.speed)	-- bruh.

							if not (mo.eflags & MFE_UNDERWATER)

								if displayplayer and displayplayer.mo and displayplayer.mo.valid
									if R_PointToDist2(mo.x, mo.y, displayplayer.mo.x, displayplayer.mo.y) < 512*mo.scale
										P_StartQuake(FRACUNIT*50, 10)
									end
								end

								local range = FRACUNIT*256

								searchBlockmap("objects", function(mo, v)
									if v and v.valid
									and R_PointToDist2(mo.x, mo.y, v.x, v.y) < 192*mo.scale
									and abs(mo.z - v.z) < 192*mo.scale
									and v.flags & MF_SHOOTABLE

										P_DamageMobj(v, mo, mo)
									end
								end, mo, mo.x - range, mo.x + range, mo.y - range, mo.y + range)

								-- visuals:
								for i = 1, 16
									local f = P_SpawnMobj(mo.x + P_RandomRange(-32, 32)*mo.scale, mo.y + P_RandomRange(-32, 32)*mo.scale, mo.z + P_RandomRange(-32, 32)*mo.scale, MT_THOK)
									f.sprite = SPR_FPRT
									f.frame = A|FF_FULLBRIGHT|FF_TRANS40
									f.scale = mo.scale
									f.destscale = mo.scale*3
									f.scalespeed = mo.scale/2

									P_InstaThrust(f, (360/16)*ANG1*i, mo.scale*32)
								end
							end
						end

						mo.silv_elmfire = nil

					end,

	icooffs = {-9, 0}
}
silv_addShieldStuff(SH_ELEMENTAL, elementaltable)

-- bad thinker for bubble...
addHook("MobjThinker", function(mo)
	if mo and mo.valid and mo.target and mo.target.valid
		if mo.target.silv_floatstate and mo.target.silv_floatstate == 2
		and abs(mo.target.momz) > 20*mo.scale

			if mo.state != S_BUBSB5
				S_StartSound(mo, sfx_s3k44)
			end

			mo.state = S_BUBSB5
		end
	end
end, MT_BUBBLEWRAP_ORB)

local bubbletable = {
	boostWater = true,
	downwardsCap = -90,
	downwardsSpeed = 5,

	boostThink = 	function(mo)

						-- yes it's still called fire because i'm too lazy to rename it.
						local fire
						if mo.eflags & MFE_VERTICALFLIP
							if mo.momz > 20*mo.scale
								fire = true
							end
						else
							if mo.momz < -20*mo.scale
								fire = true
							end
						end

						mo.silv_bubblebounce = fire	-- buffer this.
						mo.silv_bubblestrength = mo.momz
					end,

	floorTouch =	function(mo)
						local p = mo.player
						if mo.silv_bubblebounce

							mo.z = $+ 2*P_MobjFlip(mo)

							S_StartSound(mo, sfx_s3k44)

							if mo.silv_bubblestrength > 40*mo.scale
								mo.silv_bubblestrength = 40*mo.scale
							elseif mo.silv_bubblestrength < -40*mo.scale
								mo.silv_bubblestrength = -40*mo.scale
							end	-- caps

							P_InstaThrust(mo, mo.angle, mo.silv_fspeed or mo.player.speed)	-- bruh.
							mo.momz = -mo.silv_bubblestrength/2
							if mo.eflags & MFE_UNDERWATER
								mo.momz = $/2	-- water is broken.
							end

							p.charability2 = CA2_NONE
							p.pflags = $ & ~PF_SPINNING
							p.pflags = $|PF_JUMPED

							if p.pflags & PF_JUMPED	-- we jumped out?
								p.charflags = $ & ~(SF_NOJUMPDAMAGE|SF_NOJUMPSPIN)
								p.pflags = $ & ~PF_NOJUMPDAMAGE
								mo.state = S_PLAY_ROLL
								mo.silv_spinjump = true
							end
						end

						mo.silv_bubblebounce = nil
						mo.silv_bubblestrength = nil
					end,

}
silv_addShieldStuff(SH_BUBBLEWRAP, bubbletable)

addHook("MobjThinker", function(mo)
	if mo and mo.valid and mo.target and mo.target.valid
		if mo.target.silv_floatstate and mo.target.silv_floatstate == 2
		and mo.target.target and mo.target.target.valid and mo.target.target.health
			local x = mo.x - 4*cos(R_PointToAngle(mo.x, mo.y))
			local y = mo.y - 4*sin(R_PointToAngle(mo.x, mo.y))
			local s = P_SpawnMobj(x, y, mo.z, MT_THOK)
			s.sprite = SPR_MAGN
			s.frame = M
			mo.frame = M
			s.scale = mo.scale
			if mo.eflags & MFE_VERTICALFLIP
				s.eflags = $|MFE_VERTICALFLIP
			end
			s.fuse = 4
		end
	end
end, MT_ATTRACT_ORB)

freeslot("SPR_KLIT")
local attractiontable = {

	boostStart =	function(mo)
						mo.target = nil
						mo.silv_useheld = 2	-- don't home in right at the start of a boost.
						mo.silv_boostseek = nil
					end,

	boostThink = 	function(mo)
						/*for i = 1, P_RandomRange(1, 4)
							local a = P_SpawnMobj(mo.x, mo.y, mo.z +mo.height/4, MT_THOK)
							a.sprite = SPR_KLIT
							a.colorized = true
							a.color = SKINCOLOR_YELLOW
							a.scale = mo.scale/2
							a.frame = P_RandomRange(0, P) |FF_FULLBRIGHT
							a.tics = 1
							a.momx, a.momy, a.momz = mo.momx, mo.momy, mo.momz
						end*/

						--local t = silv_getTKtable()

						if not mo.target or not mo.target.valid or not mo.target.health
							mo.silv_magnetangle = mo.angle
						end

						mo.silv_useheld = $ or 0
						if mo.player.cmd.buttons & BT_USE
							mo.silv_useheld = $+1
						else
							mo.silv_useheld = 0
							mo.silv_boostseek = nil	-- you can keep holding to chain multiple enemies
						end

						local targets = {}

						if not mo.target or not mo.target.valid or not mo.target.health

							if not (mo.silv_useheld == 1 or mo.silv_boostseek) return end

							mo.silv_boostseek = true

							searchBlockmap("objects",
							function(mo, v)
								if v and v.valid and v.flags & MF_ENEMY and v.health
									if R_PointToDist2(mo.x, mo.y, v.x, v.y) <= FixedMul(mo.scale, RING_DIST)
									and abs(mo.z - v.z) <= FixedMul(mo.scale, RING_DIST)

										local moman = angtoint(R_PointToAngle2(0, 0, mo.momx, mo.momy))
										if abs((moman - angtoint(R_PointToAngle2(mo.x, mo.y, v.x, v.y)) +180) % 360 - 180) > 60
											return
										end	-- angle too steep.

										targets[#targets+1] = v
									end
								end
							end, mo, mo.x - RING_DIST*2, mo.x + RING_DIST*2, mo.y - RING_DIST*2, mo.y + RING_DIST*2)

							-- get closest target
							--print("Get new target!")
							if #targets
								if #targets > 1
									table.sort(targets, function(a, b)
										return R_PointToDist2(a.x, a.y, mo.x, mo.y) < R_PointToDist2(b.x, b.y, mo.x, mo.y)
										end)
								end

								mo.target = targets[1]

								if mo.target and mo.target.valid
									S_StartSound(mo, sfx_s3k40)
								end
							end
						end

						mo.player.pflags = $ & ~PF_SHIELDABILITY

						if mo.target and mo.target.valid and mo.target.health
							A_HomingChase(mo, min(55*mo.scale, mo.silv_fspeed))
							mo.player.pflags = $|PF_SHIELDABILITY
							mo.silv_energy = $+6	-- recover energy to not consume any.

							if mo.eflags & MFE_VERTICALFLIP
								mo.z = min($, mo.ceilingz - mo.scale*16)

								if mo.z + mo.momz > mo.ceilingz - mo.scale*16
									mo.momz = (mo.z - (mo.ceilingz - mo.scale*16))
								end
							else
								if mo.z + mo.momz < mo.floorz + mo.scale*16
									mo.momz = ((mo.floorz + mo.scale*16) - mo.z)
								end
								mo.z = max($, mo.floorz + mo.scale*16)
							end
						end
					end,
	ico = "ICO_MAGN",
	icooffs = {-10, -5},
}
silv_addShieldStuff(SH_ATTRACT, attractiontable)

local thundertable = {
	boostThink = 	function(mo)
						if leveltime%10 == 0
							S_StartSound(mo, sfx_cdfm34)
						end

						for i = 1, P_RandomRange(1, 4)
							local a = P_SpawnMobj(mo.x, mo.y, mo.z +mo.height/4, MT_THOK)
							a.sprite = SPR_KLIT
							a.colorized = true
							a.color = SKINCOLOR_WHITE
							a.scale = mo.scale/2
							a.frame = P_RandomRange(0, P) |FF_FULLBRIGHT
							a.tics = 1
							a.momx, a.momy, a.momz = mo.momx, mo.momy, mo.momz
						end

						searchBlockmap("objects",
						function(mo, v)
							if v and v.valid and v.flags & MF_ENEMY and v.health
								if R_PointToDist2(mo.x, mo.y, v.x, v.y) <= FixedMul(mo.scale, RING_DIST/2)
								and abs(mo.z - v.z) <= FixedMul(mo.scale, RING_DIST/2)
									v.target = mo
									A_HomingChase(v, 32*FRACUNIT)
									v.momx, v.momy, v.momz = $1+mo.momx, $2+mo.momy, $3+mo.momz
								end
							end
						end, mo, mo.x - RING_DIST, mo.x + RING_DIST, mo.y - RING_DIST, mo.y + RING_DIST)
					end,
	ico = "ICO_MAGN",
	icooffs = {-10, -5},
}
silv_addShieldStuff(SH_THUNDERCOIN, thundertable)

-- thinker on the flame orb for silver's boost:
addHook("MobjThinker", function(mo)

	if mo and mo.valid and mo.target and mo.target.valid
		if mo.target.silv_floatstate and mo.target.silv_floatstate == 2
			mo.angle = mo.target.angle	-- it doesn't match our angle by default, so let's force it to, lmao.
		end
	end
end, MT_FLAMEAURA_ORB)

local flametable = {
	boostSneaker = true,
	boostStart =	function(mo)
						S_StartSound(mo, sfx_s3k43)
					end,

	boostThink = 	function(mo)
						-- force the shield into boost visuals.
						mo.player.pflags = $|PF_SHIELDABILITY
					end
}
silv_addShieldStuff(SH_FLAMEAURA, flametable)